#include "stdafx.h"
#include "EditBytes.h"
#include "Imm.h"


CEditBytes::CEditBytes(): m_nLimitTextBytes(80)
, m_bImeMsgDef(false)
, m_bImeMsgDrop(false)
, m_eventFunc(NULL)
, m_eventUser(NULL)
{

}

CEditBytes::~CEditBytes()
{
}

void CEditBytes::SetEvent(EVENT_FUNC pFunc, void* pUser)
{
	m_eventFunc = pFunc;
	m_eventUser = pUser;
}

int CEditBytes::IsPossibleInput(int nByte)
{
	if ( !m_nLimitTextBytes )
		return true;

	CString strW;
	strW = m_lastStrA;

	int nLength = strW.GetLength();

	int selS, selE;
	CEdit::GetSel(selS, selE);

	strW.Delete(selS, selE-selS);

	CStringA strA;
	strA = strW;

	nLength = strA.GetLength();

	if ( (strA.GetLength() + nByte) <= m_nLimitTextBytes )
		return true;

	return false;
}


inline int GetMBLen(LPCTSTR str)
{
	return WideCharToMultiByte(CP_ACP, 0, str, -1, NULL, 0, NULL, NULL) - 1;
}

LRESULT CEditBytes::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
	case WM_IME_COMPOSITION:
		if ( m_bImeMsgDrop ) return 0;

		if ( m_bImeMsgDef ) break;

		if ( !IsPossibleInput(2) )
		{
			m_bImeMsgDrop = true;
			HIMC hIMC = ::ImmGetContext( m_hWnd );
			if ( hIMC )
			{

				::ImmSetCompositionString( hIMC, SCS_SETSTR, NULL, 0, NULL, 0 );
				::ImmReleaseContext( m_hWnd, hIMC );

			}

			m_bImeMsgDrop = false;

			if ( m_eventFunc )
				m_eventFunc(ID_MAX, m_nLimitTextBytes, m_eventUser);

			return 0;
		}

		m_bImeMsgDef = true;
		break;

	case WM_CHAR:

		if ( m_bImeMsgDef )
		{
			m_bImeMsgDef = false;
		}
		else 
		{
			if ( (wParam >= 0x20 || wParam == 0x0D || wParam == 0x0A ) && !IsPossibleInput(1) )
			{
				if ( m_eventFunc )
					m_eventFunc(ID_MAX, m_nLimitTextBytes, m_eventUser);
				return 0;
			}


		}
		break;

	case EM_REPLACESEL:
		if ( lParam )
		{
			int len = GetMBLen((LPCTSTR) lParam);
			if ( len > 0 && !IsPossibleInput(len) )
			{
				if ( m_eventFunc )
					m_eventFunc(ID_MAX, m_nLimitTextBytes, m_eventUser);
				return 0;
			}
		}
		break;

	case WM_PASTE:
		OnPaste();
		return 0;

	case WM_SETTEXT:
		if ( m_nLimitTextBytes )
		{
			if ( lParam && GetMBLen((LPCTSTR) lParam) > m_nLimitTextBytes )
				return false;

			LRESULT res = CEdit::WindowProc(message, wParam, lParam);
			OnEnChange();
			return res;
		}
		break;
	}

	return CEdit::WindowProc(message, wParam, lParam);
}

void CEditBytes::OnPaste()
{
	CString txt;

	if ( ::IsClipboardFormatAvailable(CF_TEXT) )
	{
		if ( ::OpenClipboard(m_hWnd) )
		{
			HGLOBAL hMem = ::GetClipboardData(CF_TEXT);
			if ( hMem )
			{
				char* p = (char*) ::GlobalLock(hMem);
				if ( p )
				{
					txt = p;
					GlobalUnlock(hMem);
				}
			}
			::CloseClipboard();
		}
	}

	if ( txt.GetLength() > 0 )
		CEdit::ReplaceSel(txt, true);
}
BEGIN_MESSAGE_MAP(CEditBytes, CEdit)
	ON_CONTROL_REFLECT(EN_CHANGE, &CEditBytes::OnEnChange)
END_MESSAGE_MAP()

void CEditBytes::OnEnChange()
{
	Invalidate();
	CString text;
	CEdit::GetWindowText(text);
	m_lastStrA = text;

	if ( m_eventFunc )
		m_eventFunc(ID_EDIT_CHANGE, m_lastStrA.GetLength(), m_eventUser);

}


